==> crwx-ng-0.2.0.tar.bz2: crwx-ng-0.2.0/src/crwxscreen.h <==
/***************************************************************************
 *   crwx-ng                                                               *
 *   Copyright (C) 2008 Vadim Lopatin <coolreader.org@gmail.com>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,            *
 *   MA 02110-1301, USA.                                                   *
 ***************************************************************************/

#ifndef CR_WXSCREEN_INCLUDED
#define CR_WXSCREEN_INCLUDED

#include <crgui.h>

#include <wx/wx.h>

/// wxWidget support: draw to wxImage
class CRWxScreen: public CRGUIScreenBase
{
protected:
    wxBitmap _wxbitmap;
    virtual void update(const lvRect& rc, bool full) {
        wxImage img;
        int dyy = _canvas->GetHeight();
        int dxx = _canvas->GetWidth();
        int dx = dxx;
        int dy = dyy;
        img.Create(dx, dy, true);
        unsigned char* bits = img.GetData();
        for (int y = 0; y < dy && y < dyy; y++) {
            int bpp = _canvas->GetBitsPerPixel();
            if (bpp == 32) {
                const lUInt32* src = (const lUInt32*)_canvas->GetScanLine(y);
                unsigned char* dst = bits + y * dx * 3;
                for (int x = 0; x < dx && x < dxx; x++) {
                    lUInt32 c = *src++;
                    *dst++ = (c >> 16) & 255;
                    *dst++ = (c >> 8) & 255;
                    *dst++ = (c >> 0) & 255;
                }
            } else if (bpp == 2) {
                //
                static const unsigned char palette[4][3] = {
                    { 0xff, 0xff, 0xff },
                    { 0xaa, 0xaa, 0xaa },
                    { 0x55, 0x55, 0x55 },
                    { 0x00, 0x00, 0x00 },
                };
                const lUInt8* src = (const lUInt8*)_canvas->GetScanLine(y);
                unsigned char* dst = bits + y * dx * 3;
                for (int x = 0; x < dx && x < dxx; x++) {
                    lUInt32 c = ((src[x >> 2] >> ((3 - (x & 3)) << 1))) & 3;
                    *dst++ = palette[c][0];
                    *dst++ = palette[c][1];
                    *dst++ = palette[c][2];
                }
            } else if (bpp == 1) {
                //
                static const unsigned char palette[2][3] = {
                    { 0xff, 0xff, 0xff },
                    { 0x00, 0x00, 0x00 },
                };
                const lUInt8* src = (const lUInt8*)_canvas->GetScanLine(y);
                unsigned char* dst = bits + y * dx * 3;
                for (int x = 0; x < dx && x < dxx; x++) {
                    lUInt32 c = ((src[x >> 3] >> ((7 - (x & 7))))) & 1;
                    *dst++ = palette[c][0];
                    *dst++ = palette[c][1];
                    *dst++ = palette[c][2];
                }
            }
        }

        // copy to bitmap
        wxBitmap bmp(img);
        _wxbitmap = bmp;
    }
public:
    CRWxScreen(int width, int height) : CRGUIScreenBase(width, height, true) { }
    wxBitmap getWxBitmap() {
        return _wxbitmap;
    }
};

#endif // CR_WXSCREEN_INCLUDED

==> crwx-ng-0.3.1.tar.bz2: crwx-ng-0.3.1/src/crwxscreen.h <==
/***************************************************************************
 *   crwx-ng                                                               *
 *   Copyright (C) 2008 Vadim Lopatin <coolreader.org@gmail.com>           *
 *   Copyright (C) 2023 Aleksey Chernov <valexlin@gmail.com>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,            *
 *   MA 02110-1301, USA.                                                   *
 ***************************************************************************/

#ifndef CR_WXSCREEN_INCLUDED
#define CR_WXSCREEN_INCLUDED

#include <crgui.h>

#include <wx/bitmap.h>

/// wxWidget support: draw to wxImage
class CRWxScreen: public CRGUIScreenBase
{
public:
    CRWxScreen(int width, int height) : CRGUIScreenBase(width, height, true) { }
    wxBitmap getWxBitmap() {
        return _wxbitmap;
    }
protected:
    wxBitmap _wxbitmap;
    virtual void update(const lvRect& rc, bool full);
private:
    void convert1bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
    void convert2bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
    void convert3bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
    void convert4bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
    void convert8bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
    void convert32bppData_impl(lUInt8* dstData, int width, int height, int dstPitch);
};

#endif // CR_WXSCREEN_INCLUDED
